#  You may distribute under the terms of either the GNU General Public License
#  or the Artistic License (the same terms as Perl itself)
#
#  (C) Paul Evans, 2015 -- leonerd@leonerd.org.uk

package Device::BusPirate::Chip::MPL3115A2;

use strict;
use warnings;

our $VERSION = '0.04';

=head1 NAME

C<Device::BusPirate::Chip::MPL3115A2> - deprecated

=head1 DESCRIPTION

This module has moved. it now lives at L<Device::Chip::MPL3115A2> because it
now implements the L<Device::Chip> interface.

To use it, replace

   my $pirate = Device::BusPirate->new( @pirate_args );
   my $chip = $pirate->mount_chip( "MPL3115A2" )->get;

with

   my $chip = Device::Chip::MPL3115A2->new;
   $chip->connect(
      Device::Chip::Adapter::BusPirate->new( @pirate_args )
   )->get;

Then proceed to use the C<$chip> device as before.

=cut

0x55AA;
